<?php

	class WS_Form_API_Submit extends WS_Form_API {

		public $ws_form_form;
		public $ws_form_submit;

		private $duration_server_start;
		private $spam_level;

		public function __construct() {

			// Initialize
			$this->ws_form_form = New WS_Form_Form();
			$this->ws_form_submit = new WS_Form_Submit();
			$this->duration_server_start = microtime(true);
			$this->spam_level = null;

			// Call parent on WS_Form_API
			parent::__construct();
		}

		// API - GET
		public function api_get($parameters) {

			// User capability check
			if(!WS_Form_Common::can_user('read_submission')) { return false; }

			$this->ws_form_submit->id = self::api_get_id($parameters);

			try {

				// Send JSON response
				self::api_json_response($this->ws_form_submit->db_read());

			} catch(Exception $e) { self::api_throw_error($e->getMessage()); }
		}

		// API - GET - By hash
		public function api_get_by_hash($parameters) {

			// No capabilities required, this is a public method

			// Get form hash
			$this->ws_form_submit->hash = self::api_get_hash($parameters);

			try {

				// Send JSON response
				self::api_json_response($this->ws_form_submit->db_read_by_hash(true, false, false));

			} catch(Exception $e) { self::api_throw_error($e->getMessage()); }
		}

		// API - POST
		public function api_post($parameters) {

			// No capabilities required, this is a public method

			$this->ws_form_submit->form_id = self::api_get_form_id($parameters);
			$this->ws_form_submit->hash = self::api_get_hash($parameters);

			try {

				// If hash found, look for form submission
				if($this->ws_form_submit->hash != '' && (strlen($this->ws_form_submit->hash) == 32)) {

					// Read submit by hash
					$this->ws_form_submit->db_read_by_hash();

				} else {

					// Create fresh hash for this submission
					$this->ws_form_submit->db_create_hash();
				}

				// Preview submit?
				$preview = (WS_Form_Common::get_query_var('preview', false) !== false);
				$this->ws_form_submit->preview = $preview;

				// Read  form data
				$this->ws_form_form->id = $this->ws_form_submit->form_id;
				if($preview) {

					// Draft
					$form_array = $this->ws_form_form->db_read(true, true);

				} else {

					// Published
					$form_array = $this->ws_form_form->db_read_published();

					// Form not yet published
					if($form_array === false) { self::api_throw_error_submit(__('No published form data', 'ws-form')); }
				}

				// Convert to object
				$form_object = json_decode(json_encode($form_array));

				// Spam protection - Honeypot
				$honeypot_hash = ($form_object->published_checksum != '') ? $form_object->published_checksum : 'honeypot_unpublished';
				$honeypot_value = WS_Form_Common::get_query_var("field_$honeypot_hash");
				if($honeypot_value != '') { self::api_throw_error_submit(__('Spam protection error', 'ws-form')); }

				// Get fields in single dimension array
				$fields = WS_Form_Common::get_fields_from_form($form_object);

				// Get field types in single dimension array
				$field_types = WS_Form_Config::get_field_types_flat();

				// Are we submitting the form or just saving it?
				$form_submit = (WS_Form_Common::get_query_var('submit') == 'yes');

				// Process each field
				foreach($fields as $field) {

					// If field type not specified, skip
					if(!isset($field->type)) { continue; };
					$field_type = $field->type;

					// If field type not known, skip
					if(!isset($field_types[$field_type])) { continue; };
					$field_type_config = $field_types[$field_type];

					// If field is not licensed, skip
					if(
						isset($field_type_config['pro_required']) &&
						$field_type_config['pro_required']

					) { continue; }

					// Save meta data
					if(!isset($field->id)) { continue; }
					$field_id = abs($field->id);
					$field_name = WS_FORM_FIELD_PREFIX . $field_id;

					// Field value
					$field_value = WS_Form_Common::get_query_var($field_name);

					// Field required
					$field_required = WS_Form_Common::get_object_meta_value($field, 'required', false);


					// Handle required fields
					if($form_submit && $field_required && ($field_value == '')) {

						self::api_throw_error_submit_validation(sprintf(__('Required field missing: %s', 'ws-form'), $field->label));
					}

					// If field type should not be saved, skip
					$submit_save = isset($field_type_config['submit_save']) ? $field_type_config['submit_save'] : false;

					if($submit_save !== false) {

						// Add to submit meta
						$this->ws_form_submit->meta[WS_FORM_FIELD_PREFIX . $field_id] = array('id' => $field_id, 'value' => $field_value, 'type' => $field_type);

					} else {

						// Add to submit meta protected
						$this->ws_form_submit->meta_protected[WS_FORM_FIELD_PREFIX . $field_id] = array('id' => $field_id, 'value' => $field_value, 'type' => $field_type);
					}
				}

				// Set up action for 
				add_action('wsf_actions_post_complete', array($this, 'api_post_complete'), 10, 2);

				// Actions
				if(!$this->ws_form_submit->error_validation) {

					// Determine event
					$this->ws_form_submit->event = $form_submit ? 'submit' : 'save';

					// Run actions
					try {

						do_action('wsf_actions_post', $form_object, $this->ws_form_submit, 'wsf_actions_post_complete', $this->spam_level);

					} catch(Exception $e) {

						self::api_throw_error_submit($e->getMessage());
					}

				} else {

					$action_complete_array = array();
					$action_complete_array['js'] = array();
					$action_complete_array['errors'] = array();

					// Add errors to response
					foreach($this->ws_form_submit->error_validation_messages as $message) {

						// Add to JS actions
						$action_complete_array['js'] = array(

							array(

								'action' => 'message',
								'message' => $message,
								'type' => 'danger',
								'method' => 'before',
								'duration' => 4000,
								'form_hide' => false,
								'clear' => true,
								'scroll_top' => false,
								'form_show' => false,
								'message_hide' => true
							)
						);

						// Add to debug error log
						$action_complete_array['errors'][] = $message;
					}

					// No actions should be run, so just return the submit JSON response
					self::api_post_complete($action_complete_array, false);
				}

			} catch(Exception $e) { self::api_throw_error($e->getMessage()); }
		}

		// API - POST - Complete
		public function api_post_complete($action_complete_array, $form_submit) {

			// No capabilities required, this is a public method

			// Get processing time in milliseconds
			$submit_duration_server = round((microtime(true) - $this->duration_server_start) * 1000);

			// Create response
			$json_response = ['count' => $this->ws_form_submit->count_submit, 'submit_duration_server' => $submit_duration_server, 'submit_duration_user' => $this->ws_form_submit->duration, 'submit' => $form_submit];

			// Add js to response
			if(isset($action_complete_array['js']) && is_array($action_complete_array['js']) && count($action_complete_array['js']) > 0) { $json_response['js'] = $action_complete_array['js']; }

			// Add logs to response
			if(isset($action_complete_array['logs']) && is_array($action_complete_array['logs']) && count($action_complete_array['logs']) > 0) { $json_response['logs'] = $action_complete_array['logs']; }

			// Add errors to response
			if(isset($action_complete_array['errors']) && is_array($action_complete_array['errors']) && count($action_complete_array['errors']) > 0) { $json_response['errors'] = $action_complete_array['errors']; }

			// Send response
			self::api_json_response_submit($json_response);
		}

		// Handle server side validation error
		public function api_throw_error_submit_validation($messages) {

			if(!is_array($messages)) { $messages = array($messages); }

			foreach($messages as $message) {

				$this->ws_form_submit->error_validation_messages[] = $message;
			}

			$this->ws_form_submit->error_validation = true;
		}

		// Throw API error
		public function api_throw_error_submit($message, $error_code = 400) {

			$this->ws_form_submit->error = true;
			$this->ws_form_submit->error_message = $message;
			$this->ws_form_submit->error_code = $error_code;

			self::api_json_response_submit();
		}

		// API - REPOST (This is called to repost an action)
		public function api_repost($parameters) {

			try {

				// User capability check
				if(!WS_Form_Common::can_user('edit_submission')) { return false; }

				$this->ws_form_submit->id = self::api_get_id($parameters);
				$action_index = self::api_get_action_index($parameters);

				// Read submit
				$this->ws_form_submit->db_read(true, false);

				// Preview submit?
				$preview = $this->ws_form_submit->preview;

				// Read  form data
				$this->ws_form_form->id = $this->ws_form_submit->form_id;
				if($preview) {

					// Draft
					$form_array = $this->ws_form_form->db_read(true, true);

				} else {

					// Published
					$form_array = $this->ws_form_form->db_read_published();
				}

				// Convert to object
				$form_object = json_decode(json_encode($form_array));

				// Get submit actions
				$actions = (@unserialize($this->ws_form_submit->actions) !== false) ? unserialize($this->ws_form_submit->actions) : false;
				if($actions === false) { self::api_throw_error(__('No actions found', 'ws-form')); }

				// Get action
				if(!isset($actions[$action_index])) { self::api_throw_error(__('Action index not found', 'ws-form')); }
				$action = $actions[$action_index];

				// Set up action for 
				add_action('wsf_action_repost_complete', array($this, 'api_repost_complete'), 10, 1);

				do_action('wsf_action_repost', $form_object, $this->ws_form_submit, $action, 'wsf_action_repost_complete');

			} catch(Exception $e) { self::api_throw_error($e->getMessage()); }
		}

		// API - POST - Complete
		public function api_repost_complete($return_array) {

			// User capability check
			if(!WS_Form_Common::can_user('edit_submission')) { return false; }

			// Send response
			parent::api_json_response($return_array, false, false);
		}

		// API - PUT
		public function api_put($parameters) {

			// User capability check
			if(!WS_Form_Common::can_user('edit_submission')) { return false; }

			$ws_form_submit = new WS_Form_Submit();
			$ws_form_submit->form_id = self::api_get_form_id($parameters);
			$ws_form_submit->id = self::api_get_id($parameters);

			// Get field data
			$submit = WS_Form_Common::get_query_var('submit', false, $parameters);
			if(!$submit) { return false; }

			// Put field
			$ws_form_submit->db_update_from_object($submit);

			// Send JSON response
			parent::api_json_response([], false, false);
		}

		// Handle JSON response
		public function api_json_response_submit($data = false) {

			$json_array = [];

			// Normal response
			if(!$this->ws_form_submit->error) {

				$json_array['hash'] = $this->ws_form_submit->hash;
			}

			// Set nonce
			$json_array['nonce'] = wp_create_nonce('wp_rest');

			// Set error
			$json_array['error'] = $this->ws_form_submit->error;
			$json_array['error_validation'] = $this->ws_form_submit->error_validation;

			// New data
			if($data !== false) { $json_array['data'] = $data; }

			// Set HTTP content type head
			header('Content-Type: application/json');

			// API error
			if($this->ws_form_submit->error) {

				// Set error message
				$json_array['error_message'] = $this->ws_form_submit->error_message;

				// Set response code
				switch($this->ws_form_submit->error_code) {

					case '403' :

						header('HTTP/1.1 403 Forbidden', true, 403);
						break;

					case '404' :

						header('HTTP/1.1 404 Not Found', true, 403);
						break;

					default :

						header('HTTP/1.1 400 Bad Request', true, 400);
				}
			}

			// Push JSON response
			$api_json_response = json_encode($json_array);
			if(json_last_error() !== 0) {

				// Set response code
				header('HTTP/1.1 400 Bad Request', true, 400);

				// Build error JSON
				$api_json_response = json_encode([

					'error' => 			true,
					'error_message' =>	'JSON encoding error: ' . json_last_error_msg() . ' (' . json_last_error() . ')'
				]);
			}
			echo $api_json_response;

			header("connection: close");

			// Stop execution
			exit;
		}

		// Get form ID
		public function api_get_form_id($parameters) {

			return absint(WS_Form_Common::get_query_var('id', 0));
		}

		// Get hash
		public function api_get_hash($parameters) {

			return WS_Form_Common::get_query_var('hash', 0, $parameters, true);
		}

		// Get hash
		public function api_get_action_index($parameters) {

			return WS_Form_Common::get_query_var('action_index', 0);
		}

		// Get submit ID
		public function api_get_id($parameters) {

			return absint(WS_Form_Common::get_query_var('submit_id', 0, $parameters));
		}
	}