<?php

	class WS_Form_Submit_Meta extends WS_Form_Core {

		public $id;
		public $parent_id;

		public $meta_key;
		public $meta_value;
		public $field_id;

		public $table_name;

		const DB_INSERT = 'meta_key,meta_value,field_id,parent_id';
		const DB_SELECT = 'meta_key,meta_value,field_id';

		public function __construct() {

			global $wpdb;

			$this->id = 0;
			$this->parent_id = 0;
		}

		// Get table name
		public function get_table_name() {

			global $wpdb;

			return $wpdb->prefix . WS_FORM_DB_TABLE_PREFIX . 'submit_meta';
		}

		// Read all meta data
		public function db_read_all() {

			// User capability check
			if(!WS_Form_Common::can_user('read_submission')) { return false; }

			$return_array = array();

			if($this->parent_id <= 0) { parent::db_throw_error(__('Parent ID not set')); }

			global $wpdb;

			$sql = sprintf("SELECT %s FROM %s WHERE parent_id = %u;", self::DB_SELECT, self::get_table_name(), $this->parent_id);
			$meta_array = $wpdb->get_results($sql, 'ARRAY_A');

			if($meta_array) { $return_array = $meta_array; }

			return $return_array;
		}

		// Delete
		public function db_delete() {

			// User capability check
			if(!WS_Form_Common::can_user('edit_submission')) { return false; }

			global $wpdb;

			// Read meta value to determine if this is file type (file or signature)
			$sql = sprintf("SELECT meta_value FROM %s WHERE id = %u;", self::get_table_name(), $this->id);
			$meta_value = $wpdb->get_var($sql);

			// If meta_value not found, return false, nothing to delete
			self::db_delete_file($meta_value);

			// Delete meta
			$sql = sprintf("DELETE FROM %s WHERE id = %u;", self::get_table_name(), $this->id);
			if($wpdb->query($sql) === false) { parent::db_throw_error(__('Error deleting submit meta', 'ws-form')); }
		}

		// Delete all meta in submit
		public function db_delete_by_submit() {

			// User capability check
			if(!WS_Form_Common::can_user('edit_submission')) { return false; }

			global $wpdb;

			// Read meta values to determine if any of them are a file type (file or signature)
			$sql = sprintf("SELECT meta_value FROM %s WHERE parent_id = %u;", self::get_table_name(), $this->parent_id);
			$metas = $wpdb->get_results($sql, 'ARRAY_A');
			if(is_null($metas)) { return false; }

			// Delete all files
			foreach($metas as $meta) { self::db_delete_file($meta['meta_value']); }

			// Delete submit
			$sql = sprintf("DELETE FROM %s WHERE parent_id = %u;", self::get_table_name(), $this->parent_id);
			if($wpdb->query($sql) === false) { parent::db_throw_error(__('Error deleting all submit meta', 'ws-form')); }
		}

		// Delete file associated with meta_value
		public function db_delete_file($meta_value) {

			if(is_null($meta_value)) { return false; }

			// Check to see if meta value is serialized data
			if(@unserialize($meta_value) !== false) {

				// Unserialize to get array of files
				$files = unserialize($meta_value);

				if(!is_array($files)) { return false; }

				// Run through each file
				foreach($files as $file) {

					// Read path
					if(!isset($file['path'])) { continue; }
					$path = $file['path'];

					// Read hash
					if(!isset($file['hash'])) { continue; }
					$hash = $file['hash'];

					// Read size
					if(!isset($file['size'])) { continue; }
					$size = $file['size'];

					// Get upload directory
					$file_upload_dir = wp_upload_dir()['basedir'] . '/';

					// File to delete
					$file_to_delete = $file_upload_dir . $path;

					// Check path
					if(strpos($path, $hash) === false) { parent::db_throw_error(__('Invalid file path: ', 'ws-form') . $file_to_delete); }

					// Check path does not contain rogue data
					if(strpos($file_to_delete, '..') !== false) { self::db_throw_error(__('Illegal file name', 'ws-form')); }

					// Check file exists
					if(!file_exists($file_to_delete)) { parent::db_throw_error(__('File not found: ', 'ws-form') . $file_to_delete); }

					// Check filesize
					if(filesize($file_to_delete) !== $size) { parent::db_throw_error(__('Invalid file size: ', 'ws-form') . $file_to_delete); }

					// Delete file
					if(!@unlink($file_to_delete)) { parent::db_throw_error(__('Unable to delete file: ', 'ws-form') . $file_to_delete); }
				}
			}
		}

		// Add meta data from object (Meta data is stored as an object by default to allow for JSON transfer)
		public function db_update_from_object($meta_data_object) {

			// No capabilities required, this is a public method

			self::db_update_from_array((array)$meta_data_object);
		}

		// Add meta data from array
		public function db_update_from_array($meta_data_array) {

			// No capabilities required, this is a public method

			if($this->parent_id <= 0) { parent::db_throw_error(__('Parent ID not set', 'ws-form')); }
			if(!is_array($meta_data_array)) { return true; }							// Empty data
			if(count($meta_data_array) === 0) { return true; }							// Empty data

			foreach($meta_data_array as $key => $meta_data) {

				if(is_array($meta_data)) {

					// Build meta data (field)
					$field_id = $meta_data['id'];
					$db_data = array('parent_id' => $this->parent_id, 'field_id' => $field_id, 'meta_value' => $meta_data['value'], 'meta_key' => WS_FORM_FIELD_PREFIX . $field_id);

				} else {

					// Build meta data (meta_key)
					$field_id = 0;
					$db_data = array('parent_id' => $this->parent_id, 'meta_key' => $key, 'meta_value' => $meta_data);
				}

				// Serialize arrays
				if(is_array($db_data['meta_value'])) { $db_data['meta_value'] = serialize($db_data['meta_value']); }

				global $wpdb;

				// Get ID of existing meta record
				if($field_id > 0) {

					$sql = sprintf("SELECT id FROM %s WHERE parent_id = %u AND (meta_key = '%s' OR field_id = %u) LIMIT 1", self::get_table_name(), $this->parent_id, $key, $field_id);
					$id = $wpdb->get_var($sql);
					if($id) { $db_data['id'] = $id; }
				}

				// Replace
				$replace_count = $wpdb->replace(self::get_table_name(), $db_data);
				if($replace_count === false) {

					parent::db_throw_error(__('Unable to replace meta data', 'ws-form'));
				}
			}

			return true;
		}
	}